/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/execution/ShellCommand.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.execution;

import java.io.File;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import com.netapp.common.util.CollectionUtils;

/**
 * A command line that can be run on an Operating System command shell.
 * Executed using a {@link ShellExecutor}.
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ShellCommand implements ShellRunnable {

    private List<String> command;
    private File workingDirectory;
    
    public ShellCommand(String... command) {
        this.command = new LinkedList<String>();
        this.command.addAll(Arrays.asList(command));
    }
    
    public ShellCommand(List<String> command) {
        this.command = command;
    }
    
    public List<String> getArgumentList() {
        return this.command;
    }

    public String getUserFacingCommand() {
        return this.getLogFacingCommand();
    }

    public String getLogFacingCommand() {
        return CollectionUtils.collectionToString(this.command, " ");
    }
    
    /**
     * Append arguments to this ShellCommand
     * @param arguments Args to be appended
     * @return this ShellCommand (useful for stacking calls)
     */
    public ShellCommand append(String... arguments) {
        this.command.addAll(Arrays.asList(arguments));
        return this;
    }
    
    public String getCommandString() {
        return CollectionUtils.collectionToString(this.getArgumentList(), " "); 
    }
    
    public ShellRunnable transformInto(String... argumentList) {
        return new ShellCommand(argumentList);
    }
    
    @Override
    public String toString() {
        return this.getUserFacingCommand();
    }

    public File getWorkingDirectory() {
        return this.workingDirectory;
    }

    public void setWorkingDirectory(File workingDirectory) {
        this.workingDirectory = workingDirectory;
    }
    
    @Override
    public int hashCode() {
        return new HashCodeBuilder()
            .append(this.command)
            .append(this.workingDirectory)
            .toHashCode();
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj instanceof ShellCommand) {
            if (this == obj) {
                return true;
            }
            
            ShellCommand rhs = (ShellCommand) obj;
            return new EqualsBuilder()
                .append(this.command, rhs.command)
                .append(this.workingDirectory, rhs.workingDirectory)
                .isEquals();
        }
        else {
            return false;
        }
    }
    
}
