/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/ClientAccessibleErrorProvider.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

/**
 * Exceptions thrown from a workflow that need to provide data back
 * to the client of the workflow need to implement this interface.
 * 
 * The object returned from {@link #getClientAccessibleError()}
 * (i.e. the ClientAccessibleError) will be accessible from the client
 * via an extended {@link com.netapp.common.flow.api.OperationResult}
 * 
 * The extended OperationResult must implement
 * {@link ClientAccessibleErrorContainer} so that the
 * workflow engine can populate it with the ClientAccessibleError
 * provided by this ClientAccessibleErrorProvider.
 * 
 * The ClientAccessibleError must be java Serializable.
 * The ClientAccessibleError must be serialized as part of the exception.
 * The ClientAccessibleError must be JAXB serializable.
 * 
 * @author philipc
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface ClientAccessibleErrorProvider<T> {
    
    /**
     * Returns the error that is made accessible from the workflow client.
     * 
     * Called by the worflow engine to obtain the error to make accessible to the client.
     *  
     * @return the error that is made accessible from the workflow client.
     */
    T getClientAccessibleError();
}
