/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/LifecycleListener.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

/**
 * Listener interface used to signal events that occur during the lifetime of an operation.
 * 
 * @see LifecycleAdapter
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface LifecycleListener {
    /**
     * This callback is invoked immediately after constructing a new
     * operation before it is activated for the first time.
     * @param id
     * @param clientRequestId
     */
    void operationCreated(OperationId id, String clientRequestId);
    
    /**
     * This callback is invoked when a operation is scheduled to actively execute.
     * @param id
     * @param clientRequestId
     */
    void operationActivated(OperationId id, String clientRequestId);
    
    /**
     * This callback is invoked when an operation is being put back in the PAUSED
     * queue as a result of a {@link RetryableException} or similar event.
     * @param id
     * @param clientRequestId
     */
    void operationPaused(OperationId id, String clientRequestId);
    
    /**
     * This callback is invoked when an operation has reached a terminal state.
     * @param id
     * @param clientRequestId
     */
    void operationCompleted(OperationId id, String clientRequestId);
    
    /**
     * This callback is invoked after each successful execution step.
     * @param id
     * @param clientRequestId
     * @param stepsCompleted
     * @param totalSteps
     */
    void operationProgress(OperationId id, String clientRequestId, int stepsCompleted, int totalSteps);
}
