/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/RetryableException.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow;

import java.util.Date;

/**
 * Thrown when a task wishes to be persisted and restarted from the last
 * checkpoint at a future time.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class RetryableException extends RuntimeException {
    private Date holdUntilTime;
    private Integer maxRetryAttempts;
    
    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
     * Create a default retryable exception. The time until
     * next retry will be determined by the Operation runner.
     */
    public RetryableException() {
        super();
    }
    
    /**
     * Create a retryable exception and specify the time at which
     * the operation may proceed.
     * @param holdUntil
     */
    public RetryableException(Date holdUntil) {
        super();
        this.holdUntilTime = holdUntil;
    }

    /**
     * Create a retryable exception and specify the number of seconds
     * to delay before the operation may proceed.
     * @param secondsToDelay
     */
    public RetryableException(Integer secondsToDelay) {
        this(secondsToDelay, null);
    }

    /**
     * Create a retryable exception and specify the number of seconds
     * to delay before the operation may proceed and/or the maximum
     * number of retry attempts.
     * @param secondsToDelay
     * @param maxRetryAttempts
     */
    public RetryableException(Integer secondsToDelay, Integer maxRetryAttempts) {
        super();
        setSecondsToDelay(secondsToDelay);
        setMaxRetryAttempts(maxRetryAttempts);
    }
    
    public void setHoldUntilTime(Date holdUntil) {
        this.holdUntilTime = holdUntil;
    }

    public Date getHoldUntilTime() {
        return this.holdUntilTime;
    }
    
    public void setSecondsToDelay(Integer secondsToDelay) {
        if (secondsToDelay != null) {
            Date delayTime = new Date();        
            delayTime.setTime(delayTime.getTime() + (secondsToDelay * 1000));
            setHoldUntilTime(delayTime);
        }
    }

    public Integer getMaxRetryAttempts() {
        return maxRetryAttempts;
    }

    public void setMaxRetryAttempts(Integer maxRetryAttempts) {
        this.maxRetryAttempts = maxRetryAttempts;
    }
}
