/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/flow/tasks/diag/FullSystemDump.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.flow.tasks.diag;

import java.util.List;

import com.netapp.common.flow.Task;
import com.netapp.common.flow.TaskInput;
import com.netapp.common.logging.NALogger;
import com.netapp.common.util.SysInfo;

/**
 * Create a full system dump containing 
 * - JVM properties
 * - System environment
 * - the output of a given list of OS commands
 * - a list of files
 * @author debert
 *
 */
public class FullSystemDump implements Task {
    private static final NALogger logger = NALogger.getLogger(FullSystemDump.class);
    
    private String filename;
    private String filepath;

    private String failedOperationId;
    private String failedOperationType;
    private String failedOperationClientRequestId;
    
    //    private String failedClientRequestId;
    private String scratchPath;

    private List<String> osCommands;
    private List<String> files;
    
    /**
     * @see com.netapp.common.flow.Task#execute()
     */
    public void execute() throws Exception {
        logger.internal("FullSystemDump");
        
        SysInfo sysInfo = new SysInfo();
        
        sysInfo.setOperationId(failedOperationId);
        sysInfo.setOperationType(failedOperationType);
        sysInfo.setClientRequestId(failedOperationClientRequestId);

        /*
         * open dump file
         */
        sysInfo.open();

        /*
         * dump java environment settings
         */
        sysInfo.dumpJvmInfo();
        
        /*
         * dump os environment settings
         */
        sysInfo.dumpEnv();
        
        /*
         * dump version information
         */
        sysInfo.dumpVersion();
        
        /*
         * dump all files from scratch path
         */
        sysInfo.dumpFile(scratchPath);

        /*
         * dump all log files
         */
        sysInfo.dumpLogFiles();
        
        /*
        sysInfo.dump(osCommands, files);
         */
        
        /*
         * dump all information requested by the xml configuration file
         */
        sysInfo.dumpFromXML();

        logger.internal("closing zip file");
        sysInfo.close();
    }

    /**
     * Set the filename to use.
     * @param filename
     */
    @TaskInput
    public void setFilename(String filename) {
        this.filename = filename;
    }

    /**
     * Set the base path of the file to use.
     * @param filepath
     */
    @TaskInput
    public void setFilepath(String filepath) {
        this.filepath = filepath;
    }

    /**
     * Set the commands to be executed.
     * @param osCommands
     */
    @TaskInput
    public void setOsCommands(List<String> osCommands) {
        this.osCommands = osCommands;
    }

    /**
     * Set the files to be collected.
     * @param files Fully qualified filename.
     */
    @TaskInput
    public void setFiles(List<String> files) {
        this.files = files;
    }
    
    /**
     * Set the scratch path for collecting files.
     * @param scratchPath
     */
    @TaskInput
    public void setScratchPath(String scratchPath) {
        this.scratchPath = scratchPath;
    }

    /**
     * Set the operation id of the failed operation.
     * @param failedOperationId
     */
    @TaskInput
    public void setFailedOperationId(String failedOperationId) {
        this.failedOperationId = failedOperationId;
    }
    
    /**
     * Set the operation type of the failed operation.
     * @param failedOperationType
     */
    @TaskInput
    public void setFailedOperationType(String failedOperationType) {
        this.failedOperationType = failedOperationType;
    }
    
    /**
     * Set the client request id. 
     * @param failedOperationClientRequestId
     */
    @TaskInput
    public void setFailedOperationClientRequestId(String failedOperationClientRequestId) {
        this.failedOperationClientRequestId = failedOperationClientRequestId;
    }
}
