/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/bootstrap/com/netapp/common/generator/NetappCodeFormatter.java#1 $
 * 
 * Copyright (c) 2005 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.generator;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * Default Netapp style for code generation.
 * @see NetappClassGenerator
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class NetappCodeFormatter implements CodeFormatter {
    private static final DateFormat YEAR_FORMAT = new SimpleDateFormat("yyyy");
    private static final DateFormat DATE_FORMAT = DateFormat.getDateTimeInstance();
    private static final Date NOW = new Date();

    // Trick so P4 doesn't expand this in this class
    public static final String RCS_ID = "$" + "Id" + "$";
    
    protected List<String> header = new ArrayList<String>();
    
    public NetappCodeFormatter() {
        header.add(RCS_ID);
        header.add("");
        header.add("Copyright (c) " + YEAR_FORMAT.format(NOW) + " Network Appliance, Inc.");
        header.add("All rights reserved.");
        header.add("");
        header.add("This file was generated at " + DATE_FORMAT.format(NOW));
    }

    /**
     * @see com.netapp.common.generator.CodeFormatter#tab()
     */
    public final String tab() {
        // 4 spaces
        return "    ";
    }

    /**
     * @see com.netapp.common.generator.CodeFormatter#newLine()
     */
    public final String newLine() {
        return "\n";
    }

    /**
     * @see com.netapp.common.generator.CodeFormatter#fixMe()
     */
    public final String fixMe() {
        return "// FIXME";
    }

    /**
     * @see com.netapp.common.generator.CodeFormatter#toDo()
     */
    public final String toDo() {
        return "// TODO";
    }

    /**
     * @see com.netapp.common.generator.CodeFormatter#defaultType()
     */
    public final String defaultType() {
        return "Object";
    }

    /**
     * @see com.netapp.common.generator.CodeFormatter#guid()
     */
    public String guid() {
        return "__GUID__";
    }
    
    public void addHeaderLine(String line) {
        header.add(line);
    }
    
    /**
     * Generate the header that goes at the top of every source file.
     * @return Perforce ID and copyright at the top of every source file.
     */
    public final String fileHeader() {
        StringBuilder buffer = new StringBuilder();
        
        buffer.append("/*").append(newLine());
        for (String s : header) {
            buffer.append(" * ").append(s).append(newLine());
        }
        buffer.append(" */").append(newLine());
        return buffer.toString();
    }
}
