/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/gwt/codegen/JAXBToGWTSourceTransformer.java#1 $
 * 
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.gwt.codegen;

import java.io.IOException;
import java.util.Set;
import java.util.regex.Pattern;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.annotation.processing.SupportedOptions;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.AnnotationMirror;
import javax.lang.model.element.Element;
import javax.lang.model.element.PackageElement;
import javax.lang.model.element.TypeElement;

import com.netapp.common.generator.NetappClassGenerator;

@SupportedSourceVersion(SourceVersion.RELEASE_6)
@SupportedAnnotationTypes({"javax.xml.bind.annotation.XmlType", "javax.xml.ws.WebFault"})
@SupportedOptions({"gwt.dest.dir", "gwt.package.filter"})
public class JAXBToGWTSourceTransformer extends AbstractProcessor {

    private String destDir;
    private Pattern packageFilter;
    
    @Override
    public synchronized void init(ProcessingEnvironment processingEnv) {
        super.init(processingEnv);
        
        this.destDir = this.processingEnv.getOptions().get("gwt.dest.dir");
        this.packageFilter = this.processingEnv.getOptions().get("gwt.package.filter") == null ? null : Pattern.compile(this.processingEnv.getOptions().get("gwt.package.filter"));
        if (this.destDir == null) {
            throw new IllegalArgumentException("Option gwt.dest.dir not defined");
        }
        
        System.out.printf("JAXBToGWTSourceTransformer: using destination directory %s%n", this.destDir);
    }
    
    @Override
    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnvironment) {
        
        try {
            for (TypeElement annotation : annotations) {
                for (Element element : roundEnvironment.getRootElements()) {
                    for (AnnotationMirror annotationMirror : element.getAnnotationMirrors()) {
                        if (this.processingEnv.getTypeUtils().isSameType(annotation.asType(), annotationMirror.getAnnotationType())) {
                 
                            PackageElement packageElement = this.processingEnv.getElementUtils().getPackageOf(element);
                            if (this.packageFilter != null && !this.packageFilter.matcher(packageElement.getQualifiedName()).matches()) {
                                continue;
                            }
                            
                            switch (element.getKind()) {
                                case CLASS: 
                                {
                                    TypeElement typeElement = (TypeElement) element;
                                    
                                    System.out.printf("Transforming class %s%n", typeElement.getQualifiedName().toString());
                                    
                                    GWTClassConverter gwtClassConverter = new GWTClassConverter(typeElement, this.processingEnv.getElementUtils(), this.processingEnv.getTypeUtils());
                                    this.writeCode(gwtClassConverter, packageElement, typeElement.getSimpleName() + ".java");
                                    break;
                                }
                                case INTERFACE: 
                                {
                                    TypeElement typeElement = (TypeElement) element;
                                    
                                    System.out.printf("Transforming interface %s%n", typeElement.getQualifiedName().toString());
                                    
                                    GWTInterfaceConverter gwtInterfaceConverter = new GWTInterfaceConverter(typeElement, this.processingEnv.getElementUtils(), this.processingEnv.getTypeUtils(), false);
                                    this.writeCode(gwtInterfaceConverter, packageElement, typeElement.getSimpleName() + ".java");
                                    break;
                                }
                                case ENUM:
                                {
                                    TypeElement typeElement = (TypeElement) element;
                                    
                                    System.out.printf("Transforming enum %s%n", typeElement.getQualifiedName().toString());
                                    
                                    GWTEnumConverter gwtEnumConverter = new GWTEnumConverter(typeElement, this.processingEnv.getElementUtils(), this.processingEnv.getTypeUtils());
                                    this.writeCode(gwtEnumConverter, packageElement, typeElement.getSimpleName() + ".java");
                                    break;
                                }
                                default:
                                    throw new IllegalStateException(String.format("Unexpected element kind '%s' for element '%s'", element.getKind(), element));
                            }
                        }
                    }
                }
            }
        }
        catch (Exception e) {
//            System.err.println(e);
            throw new RuntimeException(e);
        }
        return true;
    }
    
    private void writeCode(NetappClassGenerator netappClassGenerator, PackageElement packageElement, String shortClassName) throws IOException {
        netappClassGenerator.writeCode(String.format("%s/%s", this.destDir, packageElement.toString().replace('.', '/')), shortClassName);
    }
}
