/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/sunos/SolarisDeviceAnalyzer.java#1 $
 * 
 * Copyright (c) 2005 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.sunos;

import java.io.File;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.DeviceAnalyzer;
import com.netapp.common.platform.DeviceType;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.platform.RawDeviceMapping;

/**
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
class SolarisDeviceAnalyzer implements DeviceAnalyzer {
    
    private static final NALogger logger = NALogger.getLogger(SolarisDeviceAnalyzer.class);

    private static final String RAW_PREFIX = "/dev/rdsk/";
    private static final String BLOCK_PREFIX = "/dev/dsk/";
    private static final String WHOLE_DISK_SLICE = "s2";
    
    public SolarisDeviceAnalyzer() {
        super();
    }

    /**
     * @see com.netapp.dlm.services.vhi.RawDeviceMapper#lookupMappingByRawDevice(File)
     */
    public RawDeviceMapping lookupMappingByRawDevice(File rawDevice) {
        String rawDevicePath = rawDevice.getAbsolutePath();
        if (rawDevicePath.startsWith(RAW_PREFIX)) {
            File disk = new File(rawDevicePath.replaceFirst(RAW_PREFIX, BLOCK_PREFIX));
            return new RawDeviceMapping(rawDevice, disk);
        }
        else {
            logger.internal("Raw Device %s is not a valid path for this platform.", rawDevicePath);
            String error = logger.error(MsgKey.PLATFORM_RAW_DEVICE_MAPPER_UNSUPPORTED_OPERATION);
            throw new PlatformException(error);
        }
    }

    /**
     * @see com.netapp.dlm.services.vhi.RawDeviceMapper#lookupMappingByDisk(File)
     */
    public RawDeviceMapping lookupMappingByDisk(File disk) {
        String diskPath = disk.getAbsolutePath();
        if (diskPath.startsWith(BLOCK_PREFIX)) {
            File rawDevice = new File(diskPath.replaceFirst(BLOCK_PREFIX, RAW_PREFIX));
            return new RawDeviceMapping(rawDevice, disk);
        }
        else {
            logger.internal("Disk %s is not a valid path for this platform", diskPath);
            String error = logger.error(MsgKey.PLATFORM_RAW_DEVICE_MAPPER_UNSUPPORTED_OPERATION);
            throw new PlatformException(error);
        }
    }
    
    public int extractPartitionNumber(File file) {
        String path = file.getAbsolutePath();
        return Integer.parseInt(path.substring(path.length() - 1));
    }
    
    public String extractPartitionPathElement(File file) {
        String path = file.getAbsolutePath();
        return path.substring(path.lastIndexOf('s'));
    }
    
    public String extractFullDiskPathElement(File file) {
        String path = file.getAbsolutePath();
        return path.substring(0, path.lastIndexOf('s')) + WHOLE_DISK_SLICE;
    }

    public File deviceForPartition(File disk, int partitionNumber) {
        String partition = String.format("s%s", partitionNumber);
        return this.deviceForPartition(disk, partition);
    }

    public File deviceForPartition(File device, String partition) {
        return new File(device.getAbsolutePath().replaceFirst(WHOLE_DISK_SLICE + "$", partition));
    }

    public DeviceType determineDeviceType(File file) {
        String absolutePath = file.getAbsolutePath();
        if (absolutePath.startsWith(RAW_PREFIX)) {
            return DeviceType.RAW_DEVICE;
        }
        else if (absolutePath.startsWith(BLOCK_PREFIX)) {
            return absolutePath.endsWith(WHOLE_DISK_SLICE) ? DeviceType.BASE_DEVICE : DeviceType.PARTITION;
        }
        else {
            return DeviceType.NON_DEVICE;    
        }
    }
}
