/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/platform/unix/DFResult.java#1 $
 * 
 * Copyright (c) 2005 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.unix;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * Encapsulates the fields output by a 'df' command on a UNIX-based OS.
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class DFResult {

    private Map<File, FileInfo> results;
    
    public DFResult() {
        this.results = new HashMap<File, FileInfo>();
    }
    
    public FileInfo getFileInfo(File file) {
        return this.results.get(file);
    }

    public void addFileInfo(File file, FileInfo fileInfo) {
        this.results.put(file, fileInfo);
    }

    private Map<File, FileInfo> getResults() {
        return this.results;
    }
    
    /**
     * Enable accumalation of the DFResults.
     * 
     * @param dfResult
     */
    public void addResult(DFResult dfResult) {
        this.results.putAll(dfResult.getResults());
    }
    
    public static class FileInfo {
        private String fileSystem;
        private File mountPoint;
        private String filer;
        private String export;

        public FileInfo(String fileSystem, File mountPoint, String filer, String export) {
            this.fileSystem = fileSystem;
            this.mountPoint = mountPoint;
            this.filer = filer;
            this.export = export;
        }
        
        /**
         * @return The filesystem on which the file lives.  For example "agate:/vol/smo" or "/dev/sda"
         */
        public String getFileSystem() {
            return this.fileSystem;
        }
        
        /**
         * @return The mountpoint on which the file lives.
         */
        public File getMountPoint() {
            return this.mountPoint;
        }
        
        /**
         * @return The export portion of the filsystem string (or null if its not an NFS export).  For example "/vol/smo".
         */
        public String getExport() {
            return this.export;
        }
        
        /**
         * @return The filer portion of the filesystem string (or null if its not an NFS export).  For example "agate".
         */
        public String getFiler() {
            return this.filer;
        }
    }
}