/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/remote/RemoteObjectFactory.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.remote;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * Factory for creating new objects in a separate JVM.
 * The separate JVM can be on a local host or a remote host.  
 *
 * @author sirianni
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public interface RemoteObjectFactory extends Remote {

    /**
     * Instantiate a new remote object in the remote JVM
     * 
     * @param <T> Type of the remote object
     * @param clazz Class of the remote object.  Must have a no argument constructor
     * @return Reference to the new remote object
     * @throws RemoteException
     */
    <T extends Remote> T newRemoteObject(Class<T> clazz, Object... constructorArgs) throws RemoteException;
 
    /**
     * Destroy the JVM in which this RemoteObjectFactory is running
     * 
     * @throws RemoteException
     */
    void destroy() throws RemoteException;

    /**
     * Ping the RemoteObjectFactory to determine if it is still running
     * and can still be communicated with
     * 
     * @throws RemoteException If the RemoteObjectFactory is dead or any other communication error
     */
    void ping() throws RemoteException;
}
