/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/test/Assert.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.test;

import static org.junit.Assert.*;

import java.util.Collection;

import com.netapp.common.logging.MessageBundle;
import com.netapp.common.logging.MessageKey;
import com.netapp.common.util.CollectionUtils;

/**
 * Static JUnit helpers for commons based projects
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class Assert {
    private Assert() { };
    
    public static void assertMessageCode(MessageKey key, String msg) {
        assertNotNull(key);
        assertNotNull(msg);
        
        String expectedCode = MessageBundle.code(key);
        assertNotNull(expectedCode);
        
        String code = MessageBundle.code(msg);
        assertEquals(expectedCode, code);
    }
    
    public static void assertExceptionCode(MessageKey key, Throwable t) {
        assertMessageCode(key, t.getMessage());
    }
    
    /**
     * Asserts that the supplied Collection object instance is empty.
     * @param c The Collection instance to check for emptiness.
     * @see CollectionUtils#isEmpty(Collection)
     */
    public static void assertEmpty(Collection<?> c) {
        if (CollectionUtils.isEmpty(c)) {
            return;
        }

        fail("Collection was not empty, contains " + c.size() + " element(s).");
    }

    /**
     * Asserts that the supplied String object instance is NOT empty.
     * @param c The Collection instance to check for emptiness.
     * @see CollectionUtils#isEmpty(Collection)
     */
    public static void assertNotEmpty(Collection<?> c) {
        if (CollectionUtils.isEmpty(c)) {
            fail("Collection was empty.");
            return;
        }
    }

    /**
     * Asserts that this line of code was not reached
     * (i.e. an actual invocation of this method fails the JUnit test)
     */
    public static void assertNotReached() {
        fail("This line of code should not have been reached");
    }

}
