/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/bootstrap/com/netapp/common/util/ParameterizedTypeImpl.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;

/**
 * Implementation of the Generics ParameterizedType interface.
 * @see ClassUtils#parseTypeDeclaration(String)
 * 
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class ParameterizedTypeImpl implements ParameterizedType {
    private final Type rawType;
    private final Type[] actualTypeArguments;
    private final String string;
    
    ParameterizedTypeImpl(Type rawType, Type[] actualTypeArguments) {
        this.rawType = rawType;
        this.actualTypeArguments = actualTypeArguments;

        StringBuilder b = new StringBuilder();
        b.append(((Class)rawType).getName());
        
        String prefix = "<";
        for (Type t : actualTypeArguments) {
            b.append(prefix);
            if (t instanceof Class) {
                b.append(((Class)t).getName());
            }
            else {
                b.append(t.toString());
            }
            prefix = ", ";
        }
        b.append(">");
        this.string = b.toString();
    }
    
    /**
     * @see java.lang.reflect.ParameterizedType#getActualTypeArguments()
     */
    public Type[] getActualTypeArguments() {
        return this.actualTypeArguments;
    }

    /**
     * @see java.lang.reflect.ParameterizedType#getOwnerType()
     */
    public Type getOwnerType() {
        return null;
    }

    /**
     * @see java.lang.reflect.ParameterizedType#getRawType()
     */
    public Type getRawType() {
        return this.rawType;
    }
    
    @Override
    public String toString() {
        return this.string;
    }

    @Override
    public boolean equals(Object o) {
        if (!(o instanceof ParameterizedType)) {
            return false;
        }

        return o.toString().equals(this.toString());
        /*
        ParameterizedType t = (ParameterizedType) o;
        if (t.getRawType().equals(this.rawType) 
                && t.getActualTypeArguments().length == this.getActualTypeArguments().length) {
            boolean match = true;
            for (int i = 0; i < this.actualTypeArguments.length; i++) {
                match = t.getActualTypeArguments()[i].equals(this.actualTypeArguments[i]);
                if (!match) {
                    break;
                }
            }
            return match;
        }
        return false;
        */
    }
    
    @Override
    public int hashCode() {
        return this.toString().hashCode();
    }
}
