/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/util/WeakBag.java#1 $
 * 
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * Collection for weak objects.
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class WeakBag<E> implements Iterable<E> {
    private List<Reference<E>> bag = new CopyOnWriteArrayList<Reference<E>>();
    private ReferenceQueue<E> expired = new ReferenceQueue<E>();
    
    public void add(E o) {
        for (Reference<E> t : bag) {
            if (t.get() == o) {
                return;
            }
        }
        bag.add(new WeakReference<E>(o, expired));
    }
    
    public Iterator<E> iterator() {
        // See if anything expired
        Reference<? extends E> ref = expired.poll();
        while (ref != null) {
            bag.remove(ref);
            ref = expired.poll();
        }
        return new WeakBagIterator<E>(bag.iterator());
    }
    
    private static class WeakBagIterator<E> implements Iterator<E> {
        private Iterator<Reference<E>> iter;
        private E next;
        
        public WeakBagIterator(Iterator<Reference<E>> iter) {
            this.iter = iter;
            next = fixNext();
        }

        public boolean hasNext() {
            return next != null;
        }

        public E next() {
            if (next == null) {
                throw new NoSuchElementException();
            }
            E current = next;
            next = fixNext();
            return current;
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }
        
        /**
         *  Fix the next into a hard reference so it doesn't
         * get collected under us. Otherwise we can't report
         * hasNext() correctly.
         */
        private E fixNext() {
            while (iter.hasNext()) {
                E obj = iter.next().get();
                if (obj != null) {
                    return obj;
                }
            }
            return null;
        }
        
    }
}
