/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/xml/MapEntityResolver.java#1 $
 * 
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.ls.LSInput;
import org.w3c.dom.ls.LSResourceResolver;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.MessageBundle;
import com.netapp.common.logging.NALogger;

/**
 * @author sunday
 * @version $Revision: #1 $ $Date: 2010/08/20 $
 */
public class MapEntityResolver implements EntityResolver, LSResourceResolver {
    private static final NALogger logger = NALogger.getLogger(MapEntityResolver.class);
    
    private Map<String, Entry> map = new HashMap<String, Entry>();
    
    public MapEntityResolver() {
    }
    
    public void addMapping(String systemId, String resource) throws NoSuchResourceException {
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        addMapping(systemId, resource, classLoader);
    }
    
    public void addMapping(String systemId, String resource, ClassLoader classLoader) throws NoSuchResourceException {
        if (classLoader.getResource(resource) == null) {
            throw new NoSuchResourceException(MessageBundle.localizeMessage(MsgKey.XML_RESOLVER_NO_SUCH_RESOURCE, resource, systemId));
        }
        this.map.put(systemId, new Entry(resource, classLoader));
    }
    
    public InputStream getInputStream(String systemId) {
        Entry entry = map.get(systemId);
        return entry != null ? entry.getInputStream() : null;
    }
    
    public Source getSource(String systemId) {
        Entry entry = map.get(systemId);
        if (entry != null) {
            logger.internal("getSource(%s) -> %s", systemId, entry.resource);
            StreamSource source = new StreamSource(entry.getInputStream(), systemId);
            return source;
        }
        return new StreamSource(systemId);
    }
    
    /**
     * @see org.xml.sax.EntityResolver#resolveEntity(java.lang.String, java.lang.String)
     */
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException {
        Entry entry = this.map.get(systemId);
        if (entry != null) {
            logger.internal("resolveEntity(%s, %s) => %s", publicId, systemId, entry);
            return new InputSource(entry.getInputStream());
        }
        logger.internal("resolveEntity(%s, %s) UNKNOWN", publicId, systemId);
        return null;
    }
    
    public LSInput resolveResource(String type, String namespaceURI, String publicId, String systemId, String baseURI) {
        logger.internal("resolveResource(%s, %s, %s, %s, %s)", type, namespaceURI, publicId, systemId, baseURI);
        Entry entry = this.map.get(namespaceURI);
        
        if (entry == null) {
            entry = this.map.get(systemId);
        }
        if (entry == null) {
            entry = this.map.get(baseURI);
        }
        if (entry != null) {
            LSInput input = new LSInputImpl();
            input.setBaseURI(baseURI);
            input.setPublicId(publicId);
            input.setSystemId(systemId);
            input.setByteStream(entry.getInputStream());
            return input;
        }
        // TODO Auto-generated method stub
        return null;
    }
    
    private static class Entry {
        private String resource;
        private ClassLoader classLoader;
        
        protected Entry(String resource, ClassLoader classLoader) {
            this.resource = resource;
            this.classLoader = classLoader;
        }
        
        protected InputStream getInputStream() {
            return this.classLoader.getResourceAsStream(this.resource);
        }
        
        protected URL getLocalURL() {
            return this.classLoader.getResource(this.resource);
        }
        
        @Override
        public String toString() {
            return this.resource;
        }
    }
    
    private static class LSInputImpl implements LSInput {
        private String publicId;
        private String systemId;
        private String baseURI;
        private InputStream inputStream;
        
        LSInputImpl() {
        }
        
        public String getBaseURI() {
            return this.baseURI;
        }

        public InputStream getByteStream() {
            return this.inputStream;
        }

        public boolean getCertifiedText() {
            return false;
        }

        public Reader getCharacterStream() {
            return null;
        }

        public String getEncoding() {
            return null;
        }

        public String getPublicId() {
            return this.publicId;
        }

        public String getStringData() {
            return null;
        }

        public String getSystemId() {
            return this.systemId;
        }

        public void setBaseURI(String arg0) {
            this.baseURI = arg0;
        }

        public void setByteStream(InputStream arg0) {
            this.inputStream = arg0;
        }

        public void setCertifiedText(boolean arg0) {
        }

        public void setCharacterStream(Reader arg0) {
        }

        public void setEncoding(String arg0) {
        }

        public void setPublicId(String arg0) {
            this.publicId = arg0;
        }

        public void setStringData(String arg0) {
        }

        public void setSystemId(String arg0) {
            this.systemId = arg0;  
        }
        
    }
}
