/*
 * $Id: //depot/prod/newport_news/Radamant.1/src/java/com/netapp/common/zapi/generator/autozapi/AutoZapiGeneratorFactory.java#1 $
 * 
 * Copyright (c) 2009 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.zapi.generator.autozapi;

import java.util.Collection;
import java.util.Properties;

import com.netapp.common.util.ConfigProperty;
import com.netapp.common.zapi.generator.GeneratorFactory;
import com.netapp.common.zapi.parser.Api;
import com.netapp.common.zapi.parser.Category;
import com.netapp.common.zapi.parser.Errdef;
import com.netapp.common.zapi.parser.Field;
import com.netapp.common.zapi.parser.Namespace;
import com.netapp.common.zapi.parser.Type;
import com.netapp.common.zapi.parser.Typedef;
import com.netapp.common.zapi.parser.Api.ApiField;
import com.netapp.common.zapi.parser.Typedef.TypedefField;

public class AutoZapiGeneratorFactory extends GeneratorFactory<AutoZapiNamespaceGenerator, AutoZapiCategoryGenerator, AutoZapiTypeGenerator<? extends Type>, AutoZapiFieldGenerator<? extends Field>> {
    
    /** Base package name for the generated classes. */
    private final String packageName;
    private final Properties generationProperties;

    public static final String PROPERTY_MODE_VMO = "vmo"; // for VMware Orchestrator Plugin usage
    public static final ConfigProperty<String> PROPERTY_MODE = ConfigProperty.stringProperty("generationMode", "normal");
    public static final ConfigProperty<String> PROPERTY_ZAPI_TYPE_PREFIX = ConfigProperty.stringProperty("zapi.type.prefix");
    
    public AutoZapiGeneratorFactory(String packageName) {
        this(packageName, new Properties());
    }

    public AutoZapiGeneratorFactory(String packageName, Properties properties) {
        this.packageName = packageName;
        this.generationProperties = properties;
        this.generationProperties.put(PROPERTY_MODE.getProperty(), PROPERTY_MODE.getDefaultValue());
    }

    public String getProperty(String key) {
        return this.generationProperties.getProperty(key);
    }

    public Properties getProperties() {
        return this.generationProperties;
    }

    @Override
    public AutoZapiNamespaceGenerator getNamespaceGenerator(Namespace namespace) {
        return new AutoZapiNamespaceGenerator(this, namespace);
    }
    
    @Override
    public AutoZapiCategoryGenerator getCategoryGenerator(Category category) {
        return new AutoZapiCategoryGenerator(this, category);
    }
    
    @Override
    public AutoZapiTypeGenerator<? extends Type> getTypeGenerator(Type type) {
        if (type instanceof Api) {
            return new AutoZapiApiGenerator(this, (Api) type);
        }
        return new AutoZapiTypedefGenerator(this, (Typedef) type);
    }
    
    @Override
    public AutoZapiFieldGenerator<? extends Field> getFieldGenerator(Field field) {
        if (field instanceof ApiField) {
            return new AutoZapiApiFieldGenerator(this, (ApiField) field);
        }
        if (field instanceof TypedefField) {
            return new AutoZapiTypedefFieldGenerator(this, (TypedefField) field);
        }
        return new AutoZapiFieldGenerator<Field>(this, field);
    }

    public AutoZapiErrdefGenerator getErrdefGenerator(Collection<Errdef> errdefs) {
        return new AutoZapiErrdefGenerator(this.packageName, errdefs);
    }
    
    public String getPackageName() {
        return this.packageName;
    }
    
}